@@ -39,6 +39,49 @@ def _check_blob_hash(blob, info):
     assert md5_hash == info["hash"]
 
 
+def test_large_file_write_from_stream_w_user_provided_checksum(
+    shared_bucket,
+    blobs_to_delete,
+    file_data,
+):
+    blob = shared_bucket.blob(f"LargeFile{uuid.uuid4().hex}")
+
+    info = file_data["big_9MiB"]
+    with open(info["path"], "rb") as file_obj:
+        blob.upload_from_file(file_obj, crc32c_checksum_value="20tD7w==")
+        blobs_to_delete.append(blob)
+
+
+def test_large_file_write_from_stream_w_user_provided_wrong_checksum(
+    shared_bucket,
+    blobs_to_delete,
+    file_data,
+):
+    blob = shared_bucket.blob(f"LargeFile{uuid.uuid4().hex}")
+
+    info = file_data["big_9MiB"]
+    with pytest.raises(exceptions.BadRequest) as excep_info:
+        with open(info["path"], "rb") as file_obj:
+            blob.upload_from_file(file_obj, crc32c_checksum_value="A0tD7w==")
+            blobs_to_delete.append(blob)
+    assert excep_info.value.code == 400
+
+
+def test_touch_and_write_large_file_w_user_provided_checksum(
+    shared_bucket,
+    blobs_to_delete,
+    file_data,
+):
+    blob = shared_bucket.blob(f"LargeFile{uuid.uuid4().hex}")
+    blob.upload_from_string(b"")
+    blob.reload()
+
+    info = file_data["big_9MiB"]
+    with open(info["path"], "rb") as file_obj:
+        blob.upload_from_file(file_obj, crc32c_checksum_value="20tD7w==")
+        blobs_to_delete.append(blob)
+
+
 def test_large_file_write_from_stream(
     shared_bucket,
     blobs_to_delete,
@@ -425,26 +468,23 @@ def test_blob_acl_w_metageneration_match(
 
     # Exercise blob ACL with metageneration/generation match
     acl = blob.acl
-    blob.reload()
+    acl.domain("google.com").grant_read()
 
     with pytest.raises(exceptions.PreconditionFailed):
-        acl.save_predefined(
-            "publicRead", if_metageneration_match=wrong_metageneration_number
-        )
-        assert "READER" not in acl.all().get_roles()
+        acl.save(if_metageneration_match=wrong_metageneration_number)
+        assert "READER" not in acl.domain("google.com").get_roles()
 
-    acl.save_predefined("publicRead", if_metageneration_match=blob.metageneration)
-    assert "READER" in acl.all().get_roles()
+    acl.save(if_metageneration_match=blob.metageneration)
 
-    blob.reload()
-    del acl.entities["allUsers"]
+    assert "READER" in acl.domain("google.com").get_roles()
+    acl.domain("google.com").revoke_read()
 
     with pytest.raises(exceptions.PreconditionFailed):
         acl.save(if_generation_match=wrong_generation_number)
-        assert acl.has_entity("allUsers")
+        assert "READER" in acl.domain("google.com").get_roles()
 
     acl.save(if_generation_match=blob.generation)
-    assert not acl.has_entity("allUsers")
+    assert "READER" not in acl.domain("google.com").get_roles()
 
 
 def test_blob_acl_upload_predefined(
@@ -453,31 +493,32 @@ def test_blob_acl_upload_predefined(
     file_data,
     service_account,
 ):
-    control = shared_bucket.blob(f"logo{uuid.uuid4().hex}")
-    control_info = file_data["logo"]
+    control_blob = shared_bucket.blob(f"logo{uuid.uuid4().hex}")
+    control_blob_info = file_data["logo"]
 
     blob = shared_bucket.blob(f"SmallFile{uuid.uuid4().hex}")
     info = file_data["simple"]
 
     try:
-        control.upload_from_filename(control_info["path"])
+        control_blob.upload_from_filename(control_blob_info["path"])
     finally:
-        blobs_to_delete.append(control)
-
+        blobs_to_delete.append(control_blob)
     try:
-        blob.upload_from_filename(info["path"], predefined_acl="publicRead")
+        blob.upload_from_filename(info["path"], predefined_acl="private")
     finally:
         blobs_to_delete.append(blob)
 
-    control_acl = control.acl
-    assert "READER" not in control_acl.all().get_roles()
+    control_blob_acl = control_blob.acl
 
     acl = blob.acl
-    assert "READER" in acl.all().get_roles()
+    count = 0
+    for entry in acl:
+        count += 1
+        entity = entry["entity"]
+    assert count == 1
+    assert entity.lstrip("user-") == service_account.service_account_email
 
-    acl.all().revoke_read()
-    assert acl.all().get_roles() == set()
-    assert control_acl.all().get_roles() == acl.all().get_roles()
+    assert sum(1 for _ in control_blob_acl) > 1
 
 
 def test_blob_patch_metadata(
@@ -1149,3 +1190,22 @@ def test_object_retention_lock(storage_client, buckets_to_delete, blobs_to_delet
     blob.retention.retain_until_time = None
     blob.patch(override_unlocked_retention=True)
     assert blob.retention.mode is None
+
+
+def test_blob_download_as_bytes_single_shot_download(
+    shared_bucket, blobs_to_delete, file_data, service_account
+):
+    blob_name = f"download-single-shot-{uuid.uuid4().hex}"
+    info = file_data["simple"]
+    with open(info["path"], "rb") as f:
+        payload = f.read()
+
+    blob = shared_bucket.blob(blob_name)
+    blob.upload_from_string(payload)
+    blobs_to_delete.append(blob)
+
+    result_regular_download = blob.download_as_bytes(single_shot_download=False)
+    assert result_regular_download == payload
+
+    result_single_shot_download = blob.download_as_bytes(single_shot_download=True)
+    assert result_single_shot_download == payload
